#!/bin/csh -f

#-------------------------------------------------------------------------------
# SETUP AND CHECK ARGUMENTS
#-------------------------------------------------------------------------------

# Check for ATMOS_HOME environment variable
if (! ${?ATMOS_HOME}) then
  echo "Error: Environment variable ATMOS_HOME is not set"
  exit 1
endif

# Set top level variables
set ROOT = $ATMOS_HOME
set PROD = MOD_PRDS
set MDIR = $ROOT/$PROD

# Check arguments
if ($#argv != 2) then
  echo "Usage: run_modds.csh SAT FIL1KM"
  echo "where"
  echo "SAT    is the satellite name (terra or aqua)"
  echo "FIL1KM is the Level 1B 1000 meter radiance HDF file"
  exit(-1)
endif

# Extract arguments
set SAT     = $argv[1]
set FIL1KM  = `basename $argv[2]`
set DIR1KM  = `dirname  $argv[2]`

# Get platform header (MOD or MYD)
if ($SAT == "terra") then
  set HEADER = "MOD"
else if ($SAT == "aqua") then
  set HEADER = "MYD"
else
  echo "Satellite name not recognized: "$SAT
  exit(-1)
endif

#-------------------------------------------------------------------------------
# CREATE LINKS TO STATIC INPUT FILES
#-------------------------------------------------------------------------------

# Create links to static input files
ln -f -s $MDIR/coeff/destripe_config_${SAT}.dat.* .

#-------------------------------------------------------------------------------
# SET UP PCF FILE
#-------------------------------------------------------------------------------

# Get date and time string from L1B 1KM meter file
set DATE_TIME = `echo $FIL1KM:t | cut -d. -f2,3`

# Set current date/time
set CURRENT = `date -u +%Y%j%H%M%S`

# Set name of template file
set TEMPLATE = $MDIR/template/${HEADER}_PRDS.pcf.template

# Set name of new PCF file
set FILPCF = ${HEADER}_PRDS.$DATE_TIME.pcf

# Set dummy start and stop times for PCF
set START_TIME = '2000-01-01T00:00:00'
set STOP_TIME = `date -u +%Y-%m-%dT%H:%M:%S`

# Create new PCF file from the template
sed \
  -e "s?FIL1KM?${FIL1KM}?g" \
  -e "s?DIR1KM?${DIR1KM}?g" \
  -e "s?START_TIME?${START_TIME}?g" \
  -e "s?STOP_TIME?${STOP_TIME}?g" \
  $TEMPLATE >! $FILPCF

#-------------------------------------------------------------------------------
# RUN THE ALGORITHM
#-------------------------------------------------------------------------------

# Set toolkit environment variables
setenv PGS_PC_INFO_FILE ./$FILPCF
setenv PGSMSG $ROOT/toolkit/message

# Print start message
echo $PROD "processing started at "`date`

# Run the algorithm
$MDIR/bin/$PROD.exe
set PGE_STATUS = $status

# Clean up
if ($PGE_STATUS == 0) then
  rm -f $FILPCF ShmMem LogUser LogReport LogStatus
  rm -f destripe_config_${SAT}.dat.*
endif

# Print failure or success message
if ($PGE_STATUS != 0) then
  echo "Error encountered"
  exit 1
else
  echo "Successful completion"
endif

# Print finish message
echo $PROD "processing ended at  "`date`
exit 0
