module InputProducts
  !
  ! This module is used to encapsulate the information needed to read data 
  !   products from Level 2 files. We use a publicly available derived type 
  !   (inputProductInfo) with private components to store the information, and  
  !   provide several procedures for getting at the information in a controlled
  !   way. 
  !
  use hdf,           only: MAX_NC_NAME
  use ErrorHandling, only: FatalErrorMessage
  implicit none
  private
  
  character, parameter :: newLine = char(10) 
  
  type, public :: inputProductInfo
    private
    character(len = MAX_NC_NAME) :: productName
    integer                      :: productType        ! This indicates MOD04, MOD05, etc. 
    character(len = 32)          :: readLevel2PixelsAs ! Possible values are "Real", 
                                                       ! "Integer", "Bit String"
    character(len = MAX_NC_NAME) :: QADataSetName
    integer                      :: QAByte, UsefulFlagBit, &
                                    QAValueStartBit, QAValueNumBits
    logical                      :: useNighttimeData
  end type inputProductInfo
  
  public :: createInputProducts, addInputProductInfo, getInputProductInfo, &
            getInputProductNames, getInputProductTypes
contains
  ! --------------------------------------------------------------  
  function createInputProducts(inputProductNames)
    character (len = *),   dimension(:),          intent( in) :: inputProductNames
    type(inputProductInfo), dimension(size(inputProductNames)) :: createInputProducts
    ! !F90
    !
    ! !Description:
    !    Create an array of type inputProductInfo given a set of product names.
    !  
    ! !Input Parameters:
    !   inputProductNames: a list of names of the input products
    !
    ! !Output Parameters:
    !   Function returns a vector of type inputProductType that is as long
    !     as the number of names provided. 
    ! 
    ! !Revision History:
    !   $Log: InputProducts.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.3  1997/12/30  17:16:16  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.2  1997/11/06  22:54:33  pincus
    ! Added more valid values to Dervied_From_Level_2_Dataset
    !
    ! Revision 1.1  1997/10/30  17:42:04  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !    The elements of the array are nearly empty, containing only the names. 
    !
    ! !END
    
    ! Local variables
    character (len = 2048)               :: msgbuffer
    integer :: i
    
    do i = 1, size(inputProductNames)
      createInputProducts(i)%productName        = inputProductNames(i)
      createInputProducts(i)%productType        = 0
      createInputProducts(i)%readLevel2PixelsAs = ""
      createInputProducts(i)%QADataSetName      = ""
      createInputProducts(i)%QAByte             = 0
      createInputProducts(i)%UsefulFlagBit      = 0
      createInputProducts(i)%QAValueStartBit    = 0
      createInputProducts(i)%QAValueNumBits     = 0
      createInputProducts(i)%useNighttimeData   = .false.
    end do 
    
  end function createInputProducts
  ! -------------------------------------------------------------- 
  subroutine getInputProductInfo(inputProduct, productName, productType, readL2PixelsAs, &
                                 QADataSetName, QAByte, UsefulFlagBit, QAValueStartBit,  &
                                 QAValueNumBits, useNighttimeData)
    type(inputProductInfo),       intent( in) :: inputProduct
    character(len = *), optional, intent(out) :: productName
    integer,            optional, intent(out) :: productType  ! This indicates MOD04, MOD05, etc. 
    character(len = *), optional, intent(out) :: readL2PixelsAs ! Possible values are "Real", 
                                                                    ! "Integer", "Bit String"
    character(len = *), optional, intent(out) :: QADataSetName
    integer,            optional, intent(out) :: QAByte, UsefulFlagBit, &
                                                 QAValueStartBit, QAValueNumBits
    logical,            optional, intent(out) :: useNighttimeData
    ! !F90
    !
    ! !Description:
    !    Extract information from a variable of type inputProductInfo.
    !  
    ! !Input Parameters:
    !   inputProduct: a variable of type inputProductInfo from which the needed information
    !     is to be extracted.
    !
    ! !Output Parameters:
    !   productName: the name of the input product
    !   productType: index indicating the type of Level 2 file in which the dataset is
    !     found. This index is determined at runtime by the order in which the 
    !     different product types (MOD04, 05, etc.) are considered. 
    !   readL2PixelsAs: character string indicating how the data are to be read;  
    !     currently valid values are "Real", "Integer", "Bit String"
    !   QADataSetName: the name of the dataset containing the Quality Assurance information
    !     for this dataset. 
    !   QAByte: the byte of QADataSetName containing the relevant QA information 
    !   UsefulFlagBit: the bit containing the useful/not useful flag in the QA information
    !   QAValueStartBit, QAValueNumBits: the starting bit and number of bits to read to determine
    !     the QA confidence levels. 
    !   useNighttimeData: flag indicating whether or not completly dark granules should be 
    !     processed.
    ! 
    ! !Revision History:
    !   $Log: InputProducts.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.3  1997/12/30  17:16:16  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.2  1997/11/06  22:54:33  pincus
    ! Added more valid values to Dervied_From_Level_2_Dataset
    !
    ! Revision 1.1  1997/10/30  17:42:04  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !    All output arguments are optional, so only those that are supplied are 
    !      returned.
    !
    ! !END
    
    if(present(productName     )) productName      = inputProduct%productName
    if(present(productType     )) productType      = inputProduct%productType
    if(present(readL2PixelsAs  )) readL2PixelsAs   = inputProduct%readLevel2PixelsAs
    if(present(QADataSetName   )) QADataSetName    = inputProduct%QADataSetName
    if(present(QAByte          )) QAByte           = inputProduct%QAByte
    if(present(UsefulFlagBit   )) UsefulFlagBit    = inputProduct%UsefulFlagBit
    if(present(QAValueStartBit )) QAValueStartBit  = inputProduct%QAValueStartBit
    if(present(QAValueNumBits  )) QAValueNumBits   = inputProduct%QAValueNumBits
    if(present(useNighttimeData)) useNighttimeData = inputProduct%useNighttimeData
    
  end subroutine getInputProductInfo
  !
  ! -------------------------------------------------------------- 
  subroutine addInputProductInfo(inputProduct, productType, readL2PixelsAs, useNighttimeData, &
                                 QADataSetName, QAByte, UsefulFlagBit,                        &
                                 QAValueStartBit, QAValueNumBits)
    type(inputProductInfo),       intent(out) :: inputProduct
    integer,                      intent( in) :: productType  ! This indicates MOD04, MOD05, etc. 
    character(len = *),           intent( in) :: readL2PixelsAs ! Possible values are "Real", 
                                                                ! "Integer", "Bit String"
    logical,                      intent( in) :: useNighttimeData
    character(len = *), optional, intent( in) :: QADataSetName
    integer,            optional, intent( in) :: QAByte, UsefulFlagBit, &
                                                 QAValueStartBit, QAValueNumBits
    character (len = 2048)                    :: msgbuffer

    ! !F90
    !
    ! !Description:
    !    Add information to a variable of type inputProductInfo.
    !  
    ! !Input Parameters:
    !   inputProduct: a variable of type inputProductInfo from which the needed information
    !     is to be extracted.
    !   productType: index indicating the type of Level 2 file in which the dataset is
    !     found. This index is determined at runtime by the order in which the 
    !     different product types (MOD04, 05, etc.) are considered. 
    !   readL2PixelsAs: character string indicating how the data are to be read;  
    !     currently valid values are "Real", "Integer", "Bit String"
    !   QADataSetName: the name of the dataset containing the Quality Assurance information
    !     for this dataset. 
    !   QAByte: the byte of QADataSetName containing the relevant QA information 
    !   UsefulFlagBit: the bit containing the useful/not useful flag in the QA information
    !   QAValueStartBit, QAValueNumBits: the starting bit and number of bits to read to determine
    !     the QA confidence levels. 
    !   useNighttimeData: flag indicating whether or not completly dark granules should be 
    !     processed.
    !
    ! !Output Parameters:
    !   None.
    ! 
    ! !Revision History:
    !   $Log: InputProducts.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.3  1997/12/30  17:16:16  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.2  1997/11/06  22:54:33  pincus
    ! Added more valid values to Dervied_From_Level_2_Dataset
    !
    ! Revision 1.1  1997/10/30  17:42:04  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !    All QA information is required if any is to be set. 
    !
    ! !END
    
    if(any( (/ present(QADataSetName), present(QAByte), present(UsefulFlagBit), &
               present(QAValueStartBit), present(QAValueNumBits) /) )           &
       .and. .not.                                                              &
       all( (/ present(QADataSetName), present(QAByte), present(UsefulFlagBit), &
               present(QAValueStartBit), present(QAValueNumBits) /) ) ) then
            msgbuffer = "Must provide all QA information" &
                        // newLine // "Operator Action:  Notify SDST."
            call FatalErrorMessage("addInputProductInfo", msgbuffer)
     end if
       
    inputProduct%useNighttimeData = useNighttimeData 
    inputProduct%productType      = productType 
    select case(trim(readL2PixelsAs))
      case("Real", "real", "Integer", "integer", "Bit String", "bit string", "Bit_String", "bit_string")
        inputProduct%readLevel2PixelsAs = readL2PixelsAs
      case default
        msgbuffer = " Value trim(readL2PixelsAs) " // &
                    " not valid for Read_Level2_Pixels_As attribute." &
                    // newLine // "Operator Action:  Check for valid PCF file.  If wrong or " &
                    // newLine // "corrupted, stage correct PCF and rerun PGE.  Otherwise, " &
                    // newLine // "notify SDST. "
        call FatalErrorMessage("addInputProductInfo", msgbuffer)
    end select
    if(present(QADataSetName)) then
      inputProduct%QADataSetName    = QADataSetName
      inputProduct%QAByte           = QAByte
      inputProduct%UsefulFlagBit    = UsefulFlagBit
      inputProduct%QAValueStartBit  = QAValueStartBit
      inputProduct%QAValueNumBits   = QAValueNumBits
    end if
    
  end subroutine addInputProductInfo
 ! --------------------------------------------------------------  
  function getInputProductNames(inputProducts)
    type(inputProductInfo),       dimension(:), intent( in)      :: inputProducts
    character(len = MAX_NC_NAME), dimension(size(inputProducts)) :: getInputProductNames
    ! !F90
    !
    ! !Description:
    !    Returns a list of dataset names from a vector of type inputProductInfo.
    !  
    ! !Input Parameters:
    !   inputProducts: vector of type inputProductInfo from which the names are to be
    !     extracted.
    !
    ! !Output Parameters:
    !   Function returns a vector of character strings containing the data set names
    !     from each element of inputProducts
    ! 
    ! !Revision History:
    !   $Log: InputProducts.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.3  1997/12/30  17:16:16  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.2  1997/11/06  22:54:33  pincus
    ! Added more valid values to Dervied_From_Level_2_Dataset
    !
    ! Revision 1.1  1997/10/30  17:42:04  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    ! Local variables
    integer                       :: i
    character (len = MAX_NC_NAME) :: name

    do i = 1, size(inputProducts)
      call getInputProductInfo(inputProducts(i), productName = name)
      getInputProductNames(i) = name
    end do 
  end function getInputProductNames
  ! --------------------------------------------------------------  
  function getInputProductTypes(inputProducts)
    type(inputProductInfo), dimension(:), intent( in)     :: inputProducts
    integer,               dimension(size(inputProducts)) :: getInputProductTypes
    ! !F90
    !
    ! !Description:
    !    Returns a list of product indicies from a vector of type inputProductInfo.
    !  
    ! !Input Parameters:
    !   inputProducts: vector of type inputProductInfo from which the product indicies 
    !     are to be extracted.
    !
    ! !Output Parameters:
    !   Function returns an integer vector containing the indicies
    !     from each element of inputProducts. The index indicates the type of Level 2 file 
    !     in which the dataset is found, and is determined at runtime by the order 
    !     in which the different product types (MOD04, 05, etc.) are considered.
    ! 
    ! !Revision History:
    !   $Log: InputProducts.f90,v $
    !   Revision 1.1  2006/02/07 22:47:44  gumley
    !   Initial version
    !
    ! Revision 1.3  1997/12/30  17:16:16  pincus
    ! netCDF constants are now taken from the hdf module; removed reference
    ! to module netcdf. Names of the constants changed.
    !
    ! Revision 1.2  1997/11/06  22:54:33  pincus
    ! Added more valid values to Dervied_From_Level_2_Dataset
    !
    ! Revision 1.1  1997/10/30  17:42:04  pincus
    ! Initial revision
    !
    !
    ! !Team-Unique Header:
    !   Cloud Retrieval Group, NASA Goddard Space Flight Center
    !
    ! !References and Credits:
    !   Written by
    !    Robert Pincus
    !    Climate and Radiation Branch, Code 913
    !    NASA/GSFC
    !    Greenbelt MD 20771
    !    Robert.Pincus@gsfc.nasa.gov
    !
    ! !Design Notes:
    !
    ! !END
    
    ! Local variables
    integer  :: i
    integer  :: productType
    
    do i = 1, size(inputProducts)
      call getInputProductInfo(inputProducts(i), productType = productType)
      getInputProductTypes(i) = productType
    end do 
  end function getInputProductTypes
  ! --------------------------------------------------------------  
end module InputProducts
