#!/usr/local/bin/perl -w
#
# Description:
#     
#   Make_JPEG_Browse.pl uses HDFLook commands to generate a JPEG image
#   of a two-dimensional SDS from a MODIS HDF product.
#     
# Input Parameters:
#   granulepath		Path to HDF input file
#   sdsname		Name of SDS to display
#   esdtname            ESDT name of SDS jpg image file
#   sdsmin		Minimum value displayed
#   sdsmax		Maximum value displayed
#
# Output Parameters:
#   None   
#
# Externals:
#   /usr/local/bin/HDFLook
#
# Revision History:
#
#   gfireman 20041012	Initial revision, inspired by Bill Ridgway's csh scripts.
#   shi      20041108	Revised, put L2 and L3 code into different programs.
#
#
# Team-unique header:
#   This software is developed by the MODIS Science Data Support Team 
#   for the National Aeronautics and Space Administration, Goddard Space 
#   Flight Center, under contract NAS5-32373.
#
# Notes:
# 
# Examples:
#   Make_L2_JPEG_Browse.pl MYDATML2.hdf Cloud_Optical_Thickness MOBCOT 0 60
#
################################################################################
use File::Basename;
use Date::Calc qw(System_Clock Add_Delta_DHMS);
use strict;

# variables received from the caller.
#
my ($granulepath,$sdsname,$esdtname,$sdsmin,$sdsmax) = @ARGV;

my $granulefile = basename($granulepath);
my $inputdir    = dirname($granulepath);
   $inputdir ||= './';
my $outputdir   = ".";

# Make sure HDFLook and Map directory are available
#
die "HDFLook utility is not available on $ENV{Host}\n"
   unless( -x "/usr/local/bin/HDFLook");
 
#                       Generate JPEG image file 
#===============================================================================

# Generate HDFLook command file, using appropriate template

my $config_file = "L2_Browse.com";
if( $esdtname =~ /RGB/){
   open (L2_RGB, "> $config_file") || die "Can't open $config_file: $!";
   write (L2_RGB);
   close (L2_RGB);
} else{
   open (L2_BROWSE_CONFIG, "> $config_file") || die "Can't open $config_file: $!";
   write (L2_BROWSE_CONFIG);
   close (L2_BROWSE_CONFIG);
}

$ENV{'HDFLOOKMAPS'} = "/usr/local/hdflook/Maps/";
$ENV{'HDFLOOKTMP'}  = "./";
 
# Call HDFLook
my $exe_HDFLook = "/usr/local/bin/HDFLook";
system("$exe_HDFLook $config_file") && do { 
    print "E $exe_HDFLook runs with Error\n";
};

# Check output; clean up if OK
if (-s "$outputdir/$esdtname.jpg") {
  unlink $config_file, "$outputdir/$esdtname.jpg.met";
} else {
  print "Error! Output file $outputdir/$granulefile.jpg not found.\n";
}

#===============================================================================
#                       Generate JPEG metadata file 
#===============================================================================

#=============   Generate Production Date/Time  ============
# gmtime() has $year subtracted 1900, $mon,$wd,$yday are 0 based.
#
my ($sec,$min,$hour,$mday,$mon,$year,$wd, $yday) = gmtime();
                                                                                                                                                          
my $proddatetime      = sprintf("%04d%03d%02d%02d%02d",$year+1900,$yday+1,$hour,$min,$sec);
my $proddatetimef     = sprintf("%04d-%02d-%02dT%02d:%02d:%02d.000Z",
                             $year+1900,$mon+1,$mday,$hour,$min,$sec);
 
#=============   Generate LocalGranuleID        ============
# read parts from $granulefile (i.e. MOD35_L2.A2003094.0925.003.2004156010212.hdf)
#
my @iparts  = split /\./, $granulefile;

my $JPEG_filename  = "$esdtname.$iparts[1].$iparts[2].$iparts[3].$proddatetime.jpg";
#print "\n JPEG_filename = $JPEG_filename \n";

#=============   Set Collection version number  ============
my $version = $iparts[3] + 0;

#===============================================================================
#=============   Generate RangeBeginningDate/Time   ============
# read from string like "A2003094" and "0925" to get datadate and hour,minute info.
#
my $dyear = substr $iparts[1], 1,4;
my $dyday = substr $iparts[1], 5,3;
my $dhour = substr $iparts[2], 0,2;
my $dmin  = substr $iparts[2], 2,2;

my $dmon;
my $dday;
 
# add $dyday-1 days to January 1 of the same year to get the calendar month and day.
#
($dyear,$dmon,$dday) = Add_Delta_DHMS($dyear,1,1,0,0,0, $dyday -1, 0,0,0);
 
my $rangebeginningdate = sprintf("%04d-%02d-%02d",$dyear,$dmon,$dday);
my $rangebeginningtime = "$dhour:$dmin:00.000000";
 
#=============   Generate RangeEndingDate/Time   ============
# add 5 minute to the rangebeginningtime to get rangeendingdate and rangeendingtime.
#
my $Dm = 5;
($dyear,$dmon,$dday, $dhour,$dmin) = Add_Delta_DHMS($dyear,$dmon,$dday,$dhour+0,$dmin+0,0,  0,0,$Dm,0);

my $rangeendingdate = sprintf("%04d-%02d-%02d",$dyear,$dmon,$dday);
my $rangeendingtime = sprintf("%02d:%02d:00.000000",$dhour,$dmin);

#=============   Generate JPEG metadata file   ============
open(JPEG_METADATA, "> $esdtname.jpg.met") || die "failed open\n";
write(JPEG_METADATA);
close(JPEG_METADATA);

# end of code

#-------------------------------------------------------------------------------
# Notes on templates:
# 
# The "@*" fieldholder format will cause any size of output value to
# print.  "Learning Perl", 2nd Ed., O'Reilly, indicates that it should
# be on a line by itself, but I found that it works as long as it is
# the last element on a "picture" line.  Any static elements to be
# added must be included in construction of the substituted value.


#===============================================================================
#  FORMAT for generating Level 1 and Level 2 (granule) browse images 
#===============================================================================
format L2_BROWSE_CONFIG =

# Clear previous options; set paths and SDS options
clear_data
set_output_directory        @*
                            $outputdir
set_input_directory         @*
                            $inputdir
set_input_HDF_file          @*
                            $granulefile
select_SDS                  @*
                            'SDSname="'.$sdsname.'"'
set_SDS_enhance             @*
                            "min=".$sdsmin." max=".$sdsmax

# Set projection and overlays
set_projection_to_geometry  ProjectionTo="Orthographic"
add_maps                    Coastlines=yes Boundaries=No \
                            CoastlineColor="White" BoundaryColor="White"
add_grids                   color="White" \
                            LatitudeMin=-90   LatitudeMax=90   LatitudeStep=5 \
                            LongitudeMin=-180 LongitudeMax=180 LongitudeStep=5
add_latitude_labels         ForegroundColor="Red" \
                            BackgroundColor="White" \
                            Border=0 FontName="Courb.ttf" \
                            SizeFactor=1 Rotation=0 \
                            Format="%.1f" \
                            LatitudeMin=-90   LatitudeMax=90  LatitudeStep=5
add_longitude_labels        ForegroundColor="Red" \
                            BackgroundColor="White" \
                            Border=0 FontName="Courb.ttf" \
                            SizeFactor=1 Rotation=90 \
                            Format="%.1f" \
                            LongitudeMin=-180 LongitudeMax=180 LongitudeStep=5
add_scale_to_image          x=40 y=80 \
                            width=400 height=20 \
                            TickColor="White" \
                            ForegroundColor="White" \
                            BackgroundColor="Black" \
                            TextFormat="%.1f" FontName="Courb.ttf" \
                            SizeFactor=1 Rotation=90 \
                            UpLeftSide=Yes NoTicks=No

# Annotations
add_text                    x=10 y=10 \
                            ForegroundColor="White" \
                            BackgroundColor="Black" \
                            Border=0  FontName="Courb.ttf" \
                            SizeFactor=1 Rotation=0 \
                            @*
                            'Text="'.$granulefile.'"'
add_text                    x=10 y=30 \
                            ForegroundColor="White" \
                            BackgroundColor="Black" \
                            Border=0  FontName="Courb.ttf" \
                            SizeFactor=1 Rotation=0 \
                            @*
                            'Text="'.$sdsname.'"'

# Set other display options
set_image_background        color="Black"
set_misc_options            AutoAdjust=yes
set_color_scale_name        RAINBOW1

# Generate image
create_MODIS_SDS_JPEG_image @*
                            'FileName="' . $esdtname . '.jpg" \\'
                            Scaling="y=a(x-b)" width=1024 height=1024
.
# end of format "L2_BROWSE_CONFIG"

format L2_RGB  =
clear_data
set_output_directory        @*
                            $outputdir
set_input_directory         @*
                            $inputdir
set_projection_to_geometry  ProjectionTo=Orthographic
set_misc_options            AutoAdjust=yes
set_image_background        color=Black
 
set_input_hdf_file           @*
                            $granulefile
 
add_text                    x=20 y=20 ForegroundColor="White"  BackgroundColor="Black" \
                            Border=0   FontName="Courb.ttf" SizeFactor=1.0 Rotation=0 \
                            @*
                            'Text="' . $granulefile . '"'
add_maps                    Coastlines=yes Boundaries=No CoastlineColor=Green BoundaryColor=White
add_grids                   color=White LatitudeMin=-90  LatitudeMax=90 LatitudeStep= 5 \
                            LongitudeMin=-180 LongitudeMax=180 LongitudeStep=5
add_latitude_labels         ForegroundColor="Red" BackgroundColor="White" \
                            Border=0 FontName="Courb.ttf" \
                            SizeFactor=1.0 Rotation=0. Format="%.1f" \
                            LatitudeMin=-90  LatitudeMax=90 LatitudeStep=5
add_longitude_labels        ForegroundColor="Red" BackgroundColor="White" \
                            Border=0 FontName="Courb.ttf" \
                            SizeFactor=1.0 Rotation=90. Format="%.1f" \
                            LongitudeMin=-180 LongitudeMax=180 LongitudeStep=5
create_MODIS_JPEG_image      @*
                            'FileName="'. $esdtname . '.jpg" RGBMode="Autocorrelation" width=1024 height=1024 '
.
 
# end of rgb format

#===============================================================================
#  FORMAT for jpg metadata
#===============================================================================
format JPEG_METADATA =

GROUP                  = INVENTORYMETADATA
  GROUPTYPE            = MASTERGROUP

  GROUP                  = ECSDATAGRANULE

    OBJECT                 = LOCALGRANULEID
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $JPEG_filename . '"'
    END_OBJECT             = LOCALGRANULEID

    OBJECT                 = PRODUCTIONDATETIME
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $proddatetimef . '"'
    END_OBJECT             = PRODUCTIONDATETIME

  END_GROUP              = ECSDATAGRANULE

  GROUP                  = COLLECTIONDESCRIPTIONCLASS

    OBJECT                 = VERSIONID
      NUM_VAL              = 1
      VALUE                = @*
                             $version
    END_OBJECT             = VERSIONID

    OBJECT                 = SHORTNAME
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $esdtname . '"'
    END_OBJECT             = SHORTNAME

  END_GROUP              = COLLECTIONDESCRIPTIONCLASS

  END_GROUP              = COLLECTIONDESCRIPTIONCLASS

  GROUP                  = RANGEDATETIME

    OBJECT                 = RANGEENDINGDATE
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $rangeendingdate . '"'
    END_OBJECT             = RANGEENDINGDATE

    OBJECT                 = RANGEENDINGTIME

    OBJECT                 = RANGEENDINGTIME
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $rangeendingtime . '"'
    END_OBJECT             = RANGEENDINGTIME

    OBJECT                 = RANGEBEGINNINGDATE
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $rangebeginningdate . '"'
    END_OBJECT             = RANGEBEGINNINGDATE

    OBJECT                 = RANGEBEGINNINGTIME
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $rangebeginningtime . '"'
    END_OBJECT             = RANGEBEGINNINGTIME

  END_GROUP              = RANGEDATETIME

END_GROUP              = INVENTORYMETADATA

END
.
