#!/usr/local/bin/perl -w
#
# Description:
#     
#   Make_L3_JPEG_Browse.pl uses HDFLook utility to generate a JPEG image
#   of a two-dimensional SDS from a MODIS HDF product.
#     
# Input Parameters:
#   inputFile	        input file
#   sdsname		input SDS parameter
#   esdtname            ESDT of the produced jpg image file
#   sdsmin		Minimum value displayed
#   sdsmax		Maximum value displayed
#
# Output Parameters:
#   None   
#
# Externals:
#   /usr/local/bin/HDFLook
#
# Revision History:
#
#   shi	   Nov 11 2004 separated the L3 code from Make_JPEG_Browse.pl
#
# Team-unique header:
#   This software is developed by the MODIS Science Data Support Team 
#   for the National Aeronautics and Space Administration, Goddard Space 
#   Flight Center, under contract NAS5-32373.
#
# Notes:
# 
# Examples:

#   Make_L3_JPEG_Browse.pl MOD08_TL.hdf Cloud_Optical_Thickness MOBCOT 0 60
# 
#
################################################################################

use Env;
use File::Basename;
use Date::Calc qw(Add_Delta_DHMS Add_Delta_YMD); 
use strict;

# variables received from the caller.
# 
my ($inputFile,$sdsname,$esdtname,$sdsmin,$sdsmax) = @ARGV;

my $inputfile = basename($inputFile);

#=============   Generate Production Date/Time  ============
# gmtime() has $year subtracted 1900, $mon,$wd,$yday are 0 based.
#
my ($sec,$min,$hour,$mday,$mon,$year,$wd, $yday) = gmtime();
     
my $proddatetime      = sprintf("%04d%03d%02d%02d%02d",$year+1900,$yday+1,$hour,$min,$sec);
my $proddatetimef     = sprintf("%04d-%02d-%02dT%02d:%02d:%02d.000Z",
                             $year+1900,$mon+1,$mday,$hour,$min,$sec);
     
#=============   Generate LocalGranuleID        ============
my ($iesdt, $dtime, $version) = split /\./, $inputfile,4;
my $JPEG_filename  = "$esdtname.$dtime.$version.$proddatetime.jpg";
     
#print "\n JPEG_filename = $JPEG_filename \n";
     
# change 005 to just 5 for $version.
#
$version = $version + 0;
     
#=============   Generate RangeBeginningDate/Time   ============
my $dyear = substr $dtime, 1,4;
my $dyday = substr $dtime, 5,3;
my $dmon;
my $dday;
     
($dyear,$dmon,$dday) = Add_Delta_DHMS($dyear,1,1,0,0,0, $dyday -1, 0,0,0);
     
my $rangebeginningdate = sprintf("%04d-%02d-%02d",$dyear,$dmon,$dday);
my $rangebeginningtime = "00:00:00.000000";
     
if($esdtname =~ /[DE]3$/){
   my $esdt_day = ($esdtname =~ /D3$/) ? 1 : 8;
   ($dyear,$dmon,$dday) = Add_Delta_DHMS($dyear,$dmon,$dday,0,0,0,$esdt_day,0,0,0);
}elsif($esdtname =~ /M3$/){ # monthly esdts end with _M3
   ($dyear,$dmon,$dday) = Add_Delta_YMD($dyear,$dmon,$dday,0,1,0);
}else{
   die "Error: ESDT name $esdtname not one of daily, 8-day or monthly\n";
}

my $rangeendingdate = sprintf("%04d-%02d-%02d",$dyear,$dmon,$dday);
my $rangeendingtime = $rangebeginningtime;
     
#===============================================================================
#                       Generate JPEG image file
#===============================================================================
                                                                                                                                                          
# Generate HDFLook command file, using appropriate template
my $config_file = "L3_Browse.com";
open (L3_BROWSE_CONFIG, "> $config_file") || die "Can't open $config_file: $!";
write (L3_BROWSE_CONFIG);
close (L3_BROWSE_CONFIG);

# Make sure HDFLook and Map directory are available and start running
# HDFLook and metadata creation.
#
$ENV{'HDFLOOKMAPS'} = "/usr/local/hdflook/Maps/";
$ENV{'HDFLOOKTMP'}  = "./";

die "HDFLook utility is not available on $ENV{Host}\n" 
   unless( -x "/usr/local/bin/HDFLook");

# Call HDFLook
#
my $exe_HDFLook = "/usr/local/bin/HDFLook";
system "$exe_HDFLook $config_file";

# Check output; clean up if OK
#
if (-s "$esdtname.jpg") {
# unlink $config_file, "$esdtname.jpg.met";
} else {
  print "Error! Output file $esdtname.jpg not found.\n";
}

#===============================================================================
#                       Generate JPEG metadata file 
#===============================================================================
open(JPEG_METADATA, "> $esdtname.jpg.met") || die "failed open\n";
write(JPEG_METADATA);
close(JPEG_METADATA);

# end of code

#-------------------------------------------------------------------------------
# Notes on templates:
# 
# The "@*" fieldholder format will cause any size of output value to
# print.  "Learning Perl", 2nd Ed., O'Reilly, indicates that it should
# be on a line by itself, but I found that it works as long as it is
# the last element on a "picture" line.  Any static elements to be
# added must be included in construction of the substituted value.

#===============================================================================
#  FORMAT for generating Level 3 (global) browse images 
#===============================================================================
format L3_BROWSE_CONFIG =

# Clear previous options; set paths and SDS options
clear_data
set_output_directory        @*
                            "\."
set_input_directory         @*
                            "\."
set_input_HDF_file          @*
                            $inputfile
select_SDS                  @*
                            'SDSname="'.$sdsname.'"'
set_SDS_enhance             @*
                            "min=".$sdsmin." max=".$sdsmax
set_color_scale_name        RAINBOW1

# Set projection and overlays
set_projection_to_geometry  ProjectionTo="Linear"  \
                            CenterLatitudeTo=0 CenterLongitudeTo=0 WidthTo=1000 HeightTo=500 \
                            LatitudeMinTo=-90. LatitudeMaxTo=90. \
                            LongitudeMinTo=-180. LongitudeMaxTo=180.

set_misc_options            AddGranuleBorder=yes Color=black Linewidth=4

set_image_background        color=White

add_text                    x=5 y=5 ForegroundColor="RED"  BackgroundColor="WHITE" \
                            Border=0   FontName="Courb.ttf" SizeFactor=1.0 Rotation=0 \
                            @*
                            'Text="'.$sdsname.' ' . $dyear . ' ' . $dyday .'"'

add_maps                    Linewidth=2. Coastlines=yes Boundaries=Yes CoastlineColor=Black BoundaryColor=Black

add_grids                   color=Red LatitudeMin=-90  LatitudeMax=90 LatitudeStep=15 \
                            LongitudeMin=-180 LongitudeMax=180 LongitudeStep=15

add_latitude_labels         ForegroundColor="Black" BackgroundColor="White" \
                            Border=0 FontName="Courb.ttf" \
                            SizeFactor=1.0 Rotation=90. Format="%.0f" \
                            Longitude=+175 LatitudeMin=-75  LatitudeMax=75 LatitudeStep=15

add_longitude_labels        ForegroundColor="Black" BackgroundColor="White" \
                            Border=0 FontName="Courb.ttf" \
                            SizeFactor=1.0 Rotation=0. Format="%.0f" \
                            Latitude=-85 LongitudeMin=-150 LongitudeMax=150 LongitudeStep=30

add_scale_to_image          x=300 y=460 width=400 height=10 \
                            TickColor=Black ForegroundColor=Red BackgroundColor=White \
                            TextFormat="%.1f" FontName=Courb.ttf SizeFactor=1.0 \
                            UpLeftSide=Yes NoTicks=No \
                            Labels="0 computed 51 computed 102 computed 153 computed 204 computed 255 computed"

# Generate image
create_MODIS_SDS_JPEG_image  @* 
                            'FileName="'. $esdtname.'.jpg"  Scaling="y=a(x-b)"' 
.
# end of format "L3_BROWSE_CONFIG"


#===============================================================================
#  FORMAT for jpg metadata
#===============================================================================
format JPEG_METADATA =

GROUP                  = INVENTORYMETADATA
  GROUPTYPE            = MASTERGROUP

  GROUP                  = ECSDATAGRANULE

    OBJECT                 = LOCALGRANULEID
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $JPEG_filename . '"'
    END_OBJECT             = LOCALGRANULEID

    OBJECT                 = PRODUCTIONDATETIME
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $proddatetimef . '"'
    END_OBJECT             = PRODUCTIONDATETIME

  END_GROUP              = ECSDATAGRANULE

  GROUP                  = COLLECTIONDESCRIPTIONCLASS

    OBJECT                 = VERSIONID
      NUM_VAL              = 1
      VALUE                = @*
                             $version
    END_OBJECT             = VERSIONID

    OBJECT                 = SHORTNAME
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $esdtname . '"'
    END_OBJECT             = SHORTNAME

  END_GROUP              = COLLECTIONDESCRIPTIONCLASS

  END_GROUP              = COLLECTIONDESCRIPTIONCLASS

  GROUP                  = RANGEDATETIME

    OBJECT                 = RANGEENDINGDATE
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $rangeendingdate . '"'
    END_OBJECT             = RANGEENDINGDATE

    OBJECT                 = RANGEENDINGTIME

    OBJECT                 = RANGEENDINGTIME
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $rangeendingtime . '"'
    END_OBJECT             = RANGEENDINGTIME

    OBJECT                 = RANGEBEGINNINGDATE
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $rangebeginningdate . '"'
    END_OBJECT             = RANGEBEGINNINGDATE

    OBJECT                 = RANGEBEGINNINGTIME
      NUM_VAL              = 1
      VALUE                = @*
                             '"' . $rangebeginningtime . '"'
    END_OBJECT             = RANGEBEGINNINGTIME

  END_GROUP              = RANGEDATETIME

END_GROUP              = INVENTORYMETADATA

END
.
